# this module contains two dummy classifiers as classes to use as baselines

import re
import os
import numpy as np

from sklearn.dummy import DummyClassifier
from collections import Counter
from sklearn.metrics import accuracy_score

CWD = os.getcwd()
LANG_LIST = ["ru", "pl", "uk", "sl", "sk", "sr", "bg", "be", "mk", "cs", "hr", "szl", "dsb", "hsb", "cu", "rue"]


class RandomClassifier:
    """
    Dummy Random Classifier to provide a baseline.
    With calculate_accuracy, returns a random class, according to the training data distribution.
    """

    def fit(self, X, y):
        """
        Fits the RandomClassifier to a data set.

                Parameters:
                        X: feature vectors
                        y: labels
                
                Returns:
                        None
        """
        self.label_freq = Counter(y)

        self.label_probs = []               # create list of probability for each label
        for label in self.label_freq.keys():
            label_prob = self.label_freq[label]/len(X)
            self.label_probs.append(label_prob)


    def calculate_accuracy(self, X, y):
        """
        Calculates accuracy on a given set, making random label predictions based on the distribution of y.

                Parameters:
                        X: features vectors
                        y: labels
                
                Return:
                        accuracy: accuracy of predictions on the given test set
        """

        correct = []            # list of [1, 0, 0, 1, ...] where 1 = correctly classified
        for dp in y:
            y_predict = np.random.choice(LANG_LIST, p=self.label_probs)
            if y_predict == dp:
                correct.append(1)
            elif y_predict != dp:
                correct.append(0)
        
        accuracy = sum(correct)/len(correct)
        return accuracy


class MostFreqClassifier:
    """
    Dummy Most Frequent Class Classifier to provide a baseline.
    With calculate_accuracy, always returns the most frequent class.
    """

    def fit(self, X, y):
        """
        Fits the RandomClassifier to a data set.

                Parameters:
                        X: feature vectors
                        y: labels
                
                Returns:
                        None
        """

        self.label_freq = Counter(y)
        self.mfc = self.label_freq.most_common(1)[0][0]        # identify the most frequent class

    def calculate_accuracy(self, X, y):
        """
        Calculates accuracy on a given set, always predicting the most common class.

                Parameters:
                        X: features vectors
                        y: labels
                
                Return:
                        accuracy: accuracy of predictions on the given test set
        """

        correct = []            # list of [1, 0, 0, 1, ...] where 1 = correctly classified
        for dp in y:
            y_predict = self.mfc
            if y_predict == dp:
                correct.append(1)
            elif y_predict != dp:
                correct.append(0)
        
        accuracy = sum(correct)/len(correct)
        return accuracy

if __name__ == "__main__":
        # prepare training data
        with open(CWD + "/project/data/train_data.txt", "r") as f:
                train_data = f.read().strip().split("\t")

        # extract training data from train_data.txt
        X_train = []
        y_train = []
        for dp in train_data:
                with_bracket = re.search("\([^a-zA-Z]+\)", dp).group()[1:]
                no_bracket = with_bracket.strip("(").strip(")").split(", ")
                X_train.append(np.array(no_bracket, dtype=np.int32))
                y_train.append(re.search("[a-zA-Z]+", dp).group())

        # prepare test data
        with open(CWD + "/project/data/test_data.txt", "r") as f:
                test_data = f.read().strip().split("\t")

        # extract test data from test_data.txt
        X_test = []
        y_test = []
        for dp in test_data:
                with_bracket = re.search("\([^a-zA-Z]+\)", dp).group()[1:]
                no_bracket = with_bracket.strip("(").strip(")").split(", ")
                X_test.append(np.array(no_bracket, dtype=np.int32))
                y_test.append(re.search("[a-zA-Z]+", dp).group())

        # train classifiers and print accuracies
        rcf = RandomClassifier()
        rcf.fit(X_train,y_train)
        print("Random Classifier: " + str(rcf.calculate_accuracy(X_test, y_test)))

        dmcf = DummyClassifier(strategy="stratified")
        dmcf.fit(X_train,y_train)
        predictions = dmcf.predict(X_test)
        print("sklearn Random Classifier (stratified): " + str(accuracy_score(predictions, y_test)))

        dmcf = DummyClassifier(strategy="uniform")
        dmcf.fit(X_train,y_train)
        predictions = dmcf.predict(X_test)
        print("sklearn Random Classifier (uniform): " + str(accuracy_score(predictions, y_test)))

        mfcf = MostFreqClassifier()
        mfcf.fit(X_train,y_train)
        print("Most Frequent Class classifier: " + str(mfcf.calculate_accuracy(X_test, y_test)))

        mfcf = DummyClassifier(strategy="most_frequent")
        mfcf.fit(X_train,y_train)
        predictions = mfcf.predict(X_test)
        print("sklearn Most Frequent Class Classifier: " + str(accuracy_score(predictions, y_test)))
