# this module creates the data for the languages in LANG_LIST and saves it locally

import re
import collections
import time
import os
import wikipedia as wp

from sklearn.model_selection import cross_val_score
from sklearn.tree import DecisionTreeClassifier
from sklearn.ensemble import RandomForestClassifier
from sklearn.neural_network import MLPClassifier
from sklearn.naive_bayes import MultinomialNB


LANG_LIST = ["ru", "pl", "uk", "sl", "sk", "sr", "bg", "be", "mk", "cs", "hr", "szl", "dsb", "hsb", "cu", "rue"]
CWD = os.getcwd()

def generate_corpus(lang):
    '''
    Combines a random selection of wikipedia articles in a language into one text document.
    This document is then saved in the folder 'corpora' rather than returned in order to aid the development process.

            Parameters:
                    lang (str): Abbreviation for the language
            
            Returns:
                    None
    '''
    wp.set_lang(lang)
    filename = CWD + "/project/corpora/" + lang + ".txt"

    article_list = wp.random(500)    # generates random list of wikipedia article titles
    i = 0
    with open(filename, "w+", encoding="utf-8") as f:
        for article in article_list:
            try:
                article_content = wp.page(article).content
                print(i)
                i += 1
                f.write(article_content)
            except:
                print("Page content is emtpy or invalid.")


def most_common_words(lang):
    '''
    Returns list of 10 most common words in a particular text document.

            Parameters:
                    lang (str): Abbreviation for the language

            Returns:
                    top_words (list): List of 10 most common words
    '''
    filename = CWD + "/project/corpora/" + lang + ".txt"

    with open(filename, "r", encoding="utf-8") as f:
        string = f.read().lower()

    words = re.findall(r'\w+', string)      # tokenise
    top_word_freq = collections.Counter(words).most_common(10)
    top_words = []
    for word in top_word_freq:              # append just the word, not the count
        top_words.append(word[0])
    return top_words



def create_data(word_features, label):
    '''
    Creates feature vectors for a language out of randomly generated wikipedia articles, where the feature vector includes the most common words in each language and the alphabet.

            Parameters:
                    word_features (list): list of most common words in each language, based off different set of articles
                    label (str): current language 

            Returns:
                    data (list): list of values for the features in feature_list
                    y (list): list of labels, parallel to data
    '''
    wp.set_lang(lang)

    # load alphabet features
    with open(CWD + "/project/alphabets/combined_alphabet.txt", "r", encoding="utf-8") as f:
        combined_alphabet = f.read()
        alph_features = combined_alphabet.split(",")

    # create list of features
    feature_list = word_features + alph_features

    # print feature_list so this can be hard-coded (see word_features.txt)
    print(feature_list)

    y = []
    article_list = wp.random(500)           # generates random list of article titles
    i = 0
    data = []
    for article in article_list:
        datapoint = []
        print(i)
        try:
            article_content = wp.page(article).content.lower()
            i += 1

            # iterate through list of features and append count of each feature to datapoint
            for feature in feature_list:
                datapoint.append(article_content.count(feature))

            # add language label to list of labels if content extraction was successful
            y += [label]
            data.append(tuple(datapoint))
        except:
            print("Page has no content")
    return data, y

time1 = time.time()

# generate features
word_features = []
for lang in LANG_LIST:
    wp.set_lang(lang)
    print("Starting generating features for " + lang)
    generate_corpus(lang)
    word_features += most_common_words(lang)

# generate values for features
X = []
y = []
for lang in LANG_LIST:
    wp.set_lang(lang)
    print("Starting generating values for features for " + lang)
    lang_td, lang_y = create_data(word_features, lang)
    X += lang_td
    y += lang_y

with open(CWD + "/project/data/data.txt", "w+", encoding="utf-8") as f:
    for dp in zip(X, y):
        f.write(str(dp)+"\t")


time2 = time.time()
training_time = time2 - time1
print("Finished generating data. Time taken: " + str(training_time) + "\n")
print("------------------------")

# cross validation to check data
print("Cross Validation results:\n")
clf = DecisionTreeClassifier(random_state=0)
cvs = cross_val_score(clf, X, y, cv=5)
print("DecisionTreeClassifier")
print(cvs)

rf = RandomForestClassifier()
cvs = cross_val_score(rf, X, y, cv=5)
print("Random Forest")
print(cvs)

nn = MLPClassifier()
cvs = cross_val_score(nn, X, y, cv=5)
print("NN")
print(cvs)

nb = MultinomialNB()
cvs = cross_val_score(nb, X, y, cv=5)
print("Naive bayes")
print(cvs)

# for application of data generated here, see detect_language.py